package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.BillDetailEnum;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.vat.entity.BillDetail;
import pwc.taxtech.atms.vat.entity.BillDetailExample;
import pwc.taxtech.atms.vat.entity.RevenueTypeMapping;
import pwc.taxtech.atms.vat.entity.RevenueTypeMappingExample;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 *  根据收入类型配置查询开票收入开票金额总计
 */
public class KPSR extends FunctionBase implements FreeRefFunction {
    public KPSR(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 2) {
            return null;
        }

        String revenueTypeName = getStringParam(args[0], ec);//收入类型名称
        Integer billType = getIntParam(args[1], ec);//发票类型
        String formulaExpression = "KPSR(\"" + revenueTypeName + "\","+billType+")";
        logger.debug(formulaExpression);
        List<ReportCellDataSourceDto> dataSource = Lists.newArrayList();

        double result = countForTrialBalance(revenueTypeName,dataSource,billType);
        Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                FormulaDataSourceDetailType.FormulaDataSourceDto,
                new BigDecimal(result), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
        saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
        return new NumberEval(result);

    }

    private double countForTrialBalance(String revenueTypeName, List<ReportCellDataSourceDto> contain, Integer billType) {
        String queryDate = formulaContext.getYear()+(formulaContext.getPeriod()<10?("0"+formulaContext.getPeriod()):(formulaContext.getPeriod()+""));
        RevenueTypeMappingExample typeMappingExample = new RevenueTypeMappingExample();
        typeMappingExample.createCriteria().andOrgIdEqualTo(formulaContext.getOrganizationId())
                .andRevenueTypeNameEqualTo(revenueTypeName).andStartDateLessThanOrEqualTo(queryDate)
                .andEndDateGreaterThanOrEqualTo(queryDate);
        List<RevenueTypeMapping> typeMappingList = SpringContextUtil.revenueTypeMappingMapper.selectByExample(typeMappingExample);
        if (CollectionUtils.isEmpty(typeMappingList)) {
            return 0.0;
        }
        List<String> contens = typeMappingList.stream()
                .map(o -> o.getContent()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(contens)) {
            return 0.0;
        }
        BillDetailExample billDetailExample = new BillDetailExample();
        BillDetailExample.Criteria c = billDetailExample.createCriteria().andBillContentIn(contens).andProjectIdEqualTo(formulaContext.getProjectId())
                .andPeriodEqualTo(Integer.valueOf(queryDate))
                .andBillTypeEqualTo(BillDetailEnum.BillType.MAPPING.get(billType));
        List<BillDetail> billDetails = SpringContextUtil.billDetailMapper.selectByExample(billDetailExample);
        for (BillDetail billDetail : billDetails) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            dto.setAmount(billDetail.getBillAmount());
            dto.setPeriod(formulaContext.getPeriod());
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.Report.getCode());
            contain.add(dto);
        }
        return billDetails.stream().mapToDouble(a -> a.getBillAmount().doubleValue()).sum();
    }


}
